<?php

namespace FiloBlu\Storelocator\Controller;

use FiloBlu\Storelocator\Helper\Data;
use FiloBlu\Storelocator\Model\StoresFactory;
use Magento\Framework\App\ActionFactory;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\RouterInterface;
use Magento\Framework\UrlInterface;

/**
 *
 */
class Router implements RouterInterface
{
    /**
     * @var \Magento\Framework\App\ActionFactory
     */
    protected $actionFactory;

    /**
     * Helper
     *
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $_helper;

    /**
     * @var \FiloBlu\Storelocator\Model\StoresFactory
     */
    protected $_storesFactory;

    /**
     * @param \Magento\Framework\App\ActionFactory $actionFactory
     * @param \FiloBlu\Storelocator\Helper\Data $helper
     * @param \FiloBlu\Storelocator\Model\StoresFactory $storeFactory
     */
    public function __construct(
        ActionFactory $actionFactory,
        Data $helper,
        StoresFactory $storeFactory
    ) {
        $this->actionFactory = $actionFactory;
        $this->_helper = $helper;
        $this->_storesFactory = $storeFactory;
    }

    /**
     * Validate and Match Cms Page and modify request
     *
     * @param \Magento\Framework\App\RequestInterface $request
     * @return bool
     */
    public function match(RequestInterface $request)
    {
        $helper = $this->_helper;
        if ($helper->getConfig('filoblu_storelocator_section/general/enabled')) {
            $url_prefix = $helper->getConfig('filoblu_storelocator_section/general/url_prefix');
            $url_suffix = $helper->getConfig('filoblu_storelocator_section/general/url_suffix');
            $path = trim($request->getPathInfo(), '/');

            $storeModel = $this->_storesFactory->create();
            $storeModel->load($path, 'url_key');

            if ($storeModel->getId()) {
                $request->setAlias(UrlInterface::REWRITE_REQUEST_PATH_ALIAS, $path . $url_suffix);
                $request->setPathInfo('/' . 'storelocator/index/shop/id/' . $storeModel->getId());

                return $this->actionFactory->create('Magento\Framework\App\Action\Forward');
            }

            if ($url_prefix == '') {
                return $this;
            }


            if ($path == $url_prefix . $url_suffix) {
                $array = explode('/', $path);

                if (count($array) == 1) {
                    $request->setAlias(
                        UrlInterface::REWRITE_REQUEST_PATH_ALIAS,
                        $path . $url_suffix
                    );
                    $request->setPathInfo('/' . 'storelocator/index/index');

                    return $this->actionFactory->create('Magento\Framework\App\Action\Forward');
                }
            }
        }
    }
}
